classdef CCalPanel < CBlockablePanel
    %CCALPANEL NiHu.Lab Calibration Channel Selection Panel
    %
    % See also: CCalSelect, CCalSelectView, CCalibration, CBlockablePanel
    
    properties (Constant, GetAccess = private)
        Limit = 6;       % Stable limit
        TolLev = 5e-3;   % Level tolerance
        TolFreq = 5e-3;  % Frequency tolerance
    end
    
    properties (SetAccess = immutable, GetAccess = private)
        Channels    % handle to CChannels array
        player      % Sound player object
    end
    
    properties (Access = private)
        hTable  % handle to GUI table
        butAcc  % handle to Accept button
        chkSnd  % handle to Sound on/off check box
        
        FilterState % initial state of Butterworth Filter
        FilterT0    % initial time of filtered signal
    end
    
    properties (Dependent = true, SetAccess = private)
        Sel         % Selected array
    end
    
    properties (Dependent = true, Access = private)
        Accept      % Accepted array
        Stable      % Stable array
        NomSens     % nominal sensitivities
        Sens        % actual sensitivities
        Freq        % actual frequencies
        Count       % Stable counter
        isSound     % sound playback selector
    end
    
    methods
        function obj = CCalPanel(hParent, Channels)
            %CCALPANEL constructor
            
            obj@CBlockablePanel(hParent, 'Calibration Panel');
            obj.Channels = Channels;
            obj.InitPanel();
            obj.player = audioplayer(soundsignal('mav', 8000, .1), 8000);
            obj.FilterState = zeros(5,length(Channels));
            obj.FilterT0 = 0;
        end
        
        function Sel = get.Sel(obj)
            data = get(obj.hTable, 'Data');
            Sel = cell2mat(data(:,1));
        end
        
        function set.Sel(obj, sel)
            data = get(obj.hTable, 'Data');
            data(:,1) = num2cell(sel);
            set(obj.hTable, 'Data', data);
        end
        
        function Sens = get.NomSens(obj)
            data = get(obj.hTable, 'Data');
            Sens = cell2mat(data(:,3));
        end
        
        function set.NomSens(obj, NomSens)
            data = get(obj.hTable, 'Data');
            data(:,3) = num2cell(NomSens);
            set(obj.hTable, 'Data', data);
        end
        
        function Sens = get.Sens(obj)
            data = get(obj.hTable, 'Data');
            Sens = cell2mat(data(:,4));
        end
        
        function set.Sens(obj, SensIn)
            data = get(obj.hTable, 'Data');
            data(:,4) = num2cell(SensIn);
            set(obj.hTable, 'Data', data);
        end
        
        function Sens = get.Freq(obj)
            data = get(obj.hTable, 'Data');
            Sens = cell2mat(data(:,6));
        end
        
        function set.Freq(obj, FreqIn)
            data = get(obj.hTable, 'Data');
            data(:,6) = num2cell(FreqIn);
            set(obj.hTable, 'Data', data);
        end
        
        function Count = get.Count(obj)
            data = get(obj.hTable, 'Data');
            Count = cell2mat(data(:,7));
        end
        
        function set.Count(obj, Count)
            data = get(obj.hTable, 'Data');
            data(:,7) = num2cell(Count);
            set(obj.hTable, 'Data', data);
        end
        
        function Stable = get.Stable(obj)
            data = get(obj.hTable, 'Data');
            Stable = cell2mat(data(:,8));
        end
        
        function set.Stable(obj, Stab)
            data = get(obj.hTable, 'Data');
            data(:,8) = num2cell(Stab);
            set(obj.hTable, 'Data', data);
        end
        
        function Accept = get.Accept(obj)
            data = get(obj.hTable, 'Data');
            Accept = cell2mat(data(:,9));
        end
        
        function set.Accept(obj, acc)
            data = get(obj.hTable, 'Data');
            data(:,9) = num2cell(acc);
            set(obj.hTable, 'Data', data);
        end
        
        function isSound = get.isSound(obj)
            isSound = logical(get(obj.chkSnd, 'Value'));
        end
        
        function CellCallback(obj, src,event)
            %CELLCALLBACK called when a cell in the table changes
            
            row = event.Indices(1);
            if event.Indices(2) == 9
                Data = get(src, 'Data');
                if Data{row, 8} ~= true
                    Data{row, 9} = event.PreviousData;
                    set(src, 'Data', Data);
                end
            end
        end
        
        function IC = IncompatibleChannels(obj, Calibrator)
            %INCOMPATIBLECHANNELS return channels incompatible with the calibrator
            
            Ch = obj.Channels(cell2mat({obj.Channels.isSelected}));
            Ch = Ch(obj.Sel);
            IC = Ch(Calibrator.Quantity ~= Ch.getQuant());
        end
        
        function ready = Calibrate(obj, data, fs, Calibrator)
            %CALIBRATE calling this function  starts the calibration process
            
            % if there is still channel to calibrate
            if any(obj.Sel)
                % delete stable flags
                obj.Stable(obj.Sel) = false;
                % delete accepted flags
                obj.Accept(obj.Sel) = false;
                % keep relevant data
                data = data(:,obj.Sel);
                
                [Ampl, freq, obj.FilterState(:,obj.Sel), obj.FilterT0] =...
                    LevelPhase(data, fs, obj.FilterT0,...
                    Calibrator.Frequency,...
                    'butter', obj.FilterState(:,obj.Sel));
                
                % harmonic rms
                rms = abs(Ampl) / sqrt(2);
                oldSens = obj.Sens(obj.Sel);
                obj.Sens(obj.Sel) = rms ./ Calibrator.Level;
                obj.Freq(obj.Sel) = freq;
                toIncrease = obj.Sel;
                toIncrease(obj.Sel) = ...
                    abs(obj.Sens(obj.Sel)./oldSens-1) < obj.TolLev & ...
                    abs(freq' ./ Calibrator.Frequency -1) < obj.TolFreq;
                obj.Count(toIncrease) = obj.Count(toIncrease)+1;
                obj.Count(~toIncrease) = 0;
                finished = obj.Count == obj.Limit;
                if any(finished)
                    obj.PlaybackCallback();
                    set(obj.butAcc, 'BackGroundColor', 'green');
                end
                obj.Sel(finished) = false;
                obj.Stable(finished) = true;
                obj.Count(finished) = 0;
            end
            ready = ~any(obj.Sel);
        end
        
        function AcceptCallback(obj)
            %ACCEPTCALLBACK Callback function of Accept button
            
            if ~any(obj.Accept & obj.Stable)
                warndlg('Select accepted stable channels before pressing the Accept button',...
                    'Nothing accepted', 'modal');
                return;
            end
            ChSel = cell2mat({obj.Channels.isSelected});
            Ch = obj.Channels(ChSel);
            Ch = Ch(obj.Accept & obj.Stable);
            S = obj.Sens(obj.Accept & obj.Stable);
            for i = 1 : length(Ch)
                Ch(i).Sensor.ActualSensitivity = S(i);
            end
            set(obj.butAcc, 'BackGroundColor', get(obj.butAcc, 'userData'));
        end
        
        function PlaybackCallback(obj)
            %PLAYBACKCALLBACK Callback function of playback button
            if obj.isSound
                play(obj.player);
            end
        end
        
        function RefreshTable(obj)
            %REFRESHTABLE called when table data is changed
            
            % Draw big table
            ChSel = cell2mat({obj.Channels.isSelected});
            Chan = obj.Channels(ChSel);
            rowname = {Chan.PhysChNr};
            nCh = length(Chan);
            dat = cell(nCh, 9);
            for iCh = 1 : nCh
                dat(iCh,:) = {false, Chan(iCh).Name,...
                    Chan(iCh).Sensor.NominalSensitivity,...
                    Chan(iCh).Sensor.ActualSensitivity,...
                    sprintf('V/%s', Chan(iCh).Sensor.Quantity.EU), 0, 0, false, false};
            end
            set(obj.hTable, 'Data', dat, 'RowName', rowname);
            set(obj.hTable, 'Position', extent(obj.hTable));
            fun = get(obj.hPanel, 'ResizeFcn');
            if ~isempty(fun)
                feval(fun);
            end
            fun = get(gcf, 'ResizeFcn');
            if ~isempty(fun)
                feval(fun);
            end
        end
        
    end % of public methods
    
    methods (Access = private)
        function InitPanel(obj)
            obj.chkSnd = uicontrol('parent', obj.hPanel,...
                'Style', 'CheckBox', ...
                'Value', true,...
                'units', 'pixels', ...
                'position', [0 0 CSetupClass.itemHeight CSetupClass.itemHeight]);
            butSnd = uicontrol('parent', obj.hPanel, 'style', 'pushbutton', ...
                'String', 'Playback', 'units', 'pixels', ...
                'position', [0 0 CSetupClass.itemWidth CSetupClass.itemHeight], ...
                'callback', @(src,event)PlaybackCallback(obj));
            obj.butAcc = uicontrol('parent', obj.hPanel, 'style', 'pushbutton', ...
                'String', 'Accept', 'units', 'pixels', ...
                'position', [0 0 CSetupClass.itemWidth CSetupClass.itemHeight], ...
                'Callback', @(src,event)AcceptCallback(obj));
            set(obj.butAcc, 'userData', get(obj.butAcc, 'BackGroundColor'));
            
            columnname = {'Sel.', 'Name', 'Nom. Sens.', 'Sens.', 'EU', 'Freq', ...
                'Cnt', 'Stab.', 'Acc.'};
            columnwidth = {30, 100, 80, 80, 30, 60, 30, 30, 30};
            columnformat = {'logical', 'char', 'short eng', 'short eng',...
                'char', 'numeric', 'numeric', 'logical', 'logical'};
            columneditable =  [true false false false false false false false true];
            obj.hTable = uitable('parent', obj.hPanel, 'Units','Pixels',...
                'ColumnName', columnname,...
                'ColumnFormat', columnformat, 'ColumnWidth', columnwidth,...
                'ColumnEditable', columneditable, ...
                'CellEditCallback', @(src,event)CellCallback(obj, src, event));
            
            RefreshTable(obj);
            
            lbar = {{{{butSnd, [0 0]}, {[], [-1 0]}, {obj.chkSnd, [0 0]}}, [0 0]}; {obj.butAcc, [-1 0]}};
            Layout = {{obj.hTable, [0, 0]}, {lbar, [0 0]}};
            GuiLayout.ApplyLayout(obj.hPanel, Layout);
        end
    end % of private methods
end % of classdef
