classdef CCalibration < CUiTab & CChSetupSensitive
    %CCALIBRATION Nihu.Lab module for sensor calibration
    %
    % See also: ChannelSetup, CMonitor, CRecord, CGuiModule
    
    properties (SetAccess = immutable, GetAccess = private)
        Hardware        % handle to CHardware object
        
        AcqSetup        % handle to CAcqSetup class
        AcqSetupView    % handle to CQcqSetupView class
        CalSelect       % handle to CCalSelect class
        CalSelectView   % handle to CCalSelectView class
        LevelMeter      % handle to CLevelMeter class
        TimeData        % handle to CGraphData object
        TimeDataView    % handle to CGraphDataView object
        FreqData        % handle to CGraphData object
        FreqDataView    % handle to CGraphDataView object
        CalPanel        % handle to CCalpanel class
    end
    
    properties (SetAccess = private)
        isRunning       % boolean indicating whether Monitoring is on
    end
    
    methods
        function obj = CCalibration(ParentGroup, hardware, chsetup, acqsetup, calselect, levmeter)
            obj = obj@CUiTab(ParentGroup, 'CALIBRATION_MODULE');
            obj = obj@CChSetupSensitive(chsetup);
            
            obj.Hardware = hardware;
            obj.AcqSetup = acqsetup;
            obj.AcqSetupView = CAcqSetupView(obj.hContainer, obj.AcqSetup);
            obj.CalSelect = calselect;
            obj.CalSelectView = CCalSelectView(obj.hContainer, obj.CalSelect);
            obj.LevelMeter = levmeter;
            obj.TimeData = CGraphData;
            obj.TimeDataView = CGraphDataView(obj.hContainer, obj.TimeData, 'INVOLTAGETIME');
            obj.FreqData = CGraphData;
            obj.FreqDataView = CGraphDataView(obj.hContainer, obj.FreqData, 'INVOLTAGESPECT');
            obj.CalPanel = CCalPanel(obj.hContainer, obj.ChannelSetup.InputChannels);
            
            obj.InitFigure();
        end
        
        function CalibCallback(obj, hButton)
            %CALIBCALLBACK Callback function for Calibration Start/Stop button
            
            % Check if a channel has been selected for calibration
            if ~obj.isRunning && ~any(obj.CalPanel.Sel)
                warndlg(CBabel.getInstance().getText('CALCHANNELSELECT_WARNING'), ...
                    CBabel.getInstance().getText('CAL_WARNING'), 'modal');
                return;
            end
            
            % Check if all selected channels are compatible with the calibrator
            if ~obj.isRunning
                IC = obj.CalPanel.IncompatibleChannels(obj.CalSelect.Calibrator);
                if ~isempty(IC)
                    str = sprintf(CBabel.getInstance().getText('CAL_INCOMPATIBLE_ERROR'), ...
                        sprintf('%s\n', IC.Name));
                    errordlg(str, CBabel.getInstance().getText('CAL_WARNING'), 'modal');
                    return;
                end
            end
            
            % switch isRunning status
            obj.isRunning = ~obj.isRunning;
            
            % If calibration has stopped, nothing needs to be done
            if ~obj.isRunning
                return
            end
            
            % start calibration
            set(hButton, 'String', CBabel.getInstance().getText('STOP_CALIBRATION'));
            set(gcbf, 'UserData', get(gcbf, 'CloseRequestFcn'));
            set(gcbf, 'CloseRequestFcn', '');
            obj.hParentGroup.block();
            obj.AcqSetupView.block();   % block Acquisition setup
            obj.CalSelectView.block();  % block Calibrator selection
            obj.CalPanel.block();       % block Calibration channel selection
            
            % time and frequency scales of plots
            tScale = (0:obj.AcqSetup.nSamples-1)'/obj.AcqSetup.Fs;
            fScale = (0:obj.AcqSetup.nSamples-1)'*obj.AcqSetup.Fs/obj.AcqSetup.nSamples;
            fScale = fScale(fScale <= obj.AcqSetup.Fs/2);
            try
                % Start hardware
                status = obj.Hardware.MonitorStart(obj.AcqSetup.Fs, obj.AcqSetup.nSamples);
                if status < 0
                    error('NiHuLab:hardware', 'Hardware start error');
                end
                while true
                    if ~obj.isRunning
                        break
                    end
                    
                    % Acquire data
                    [status, voltage] = obj.Hardware.MonitorAcq(obj.AcqSetup.nSamples);
                    if status < 0
                        error('NiHuLab:hardware', 'Hardware acquisition error');
                    end
                    % display levels
                    obj.LevelMeter.UpdateLevels(voltage);
                    % display input voltage time history
                    obj.TimeData.UpdateData(tScale, voltage);
                    Voltage = fft(voltage, [], 1);
                    obj.FreqData.UpdateData(fScale, abs(Voltage(1:length(fScale),:)));
                    
                    % Compute voltage at the input of preamplifiers
                    voltage = obj.ChannelSetup.decompensate_preamp(voltage);
                    % calibrate using preamp voltage
                    ok = obj.CalPanel.Calibrate(voltage, obj.AcqSetup.Fs, obj.CalSelect.Calibrator);
                    if ok
                        break
                    end
                    drawnow;
                end
                % stop calibration
                status = obj.Hardware.MonitorStop();
                if status < 0
                    error('NiHuLab:hardware', 'Hardware closing error');
                end
            catch ME
                errordlg(sprintf('%s\n%s', ME.identifier, ME.message), '!!! Error !!!', 'modal');
            end
            
            set(hButton, 'String', CBabel.getInstance().getText('START_CALIBRATION'));
            set(gcbf, 'CloseRequestFcn', get(gcbf, 'UserData'));
            obj.hParentGroup.unblock();
            obj.AcqSetupView.unblock();     % allow acquisition setup
            obj.CalSelectView.unblock();    % allow calibrator selection
            obj.CalPanel.unblock();         % allow calibration channel selection
            % if error has occurred, we must indicate that calibration is stopped
            obj.isRunning = false;
        end
    end
    
    methods
        function show(obj)
            %SHOW function called each time before the module is opened
            
            % Refresh calibrator panel based on a possibly new Channel Setup
            obj.CalPanel.RefreshTable();
            
            obj.isRunning = false;
            
            obj.LevelMeter.Reset();
            
            if obj.chSetupChanged
                % update Time and Frequency plots with new Channel setup
                Chan = obj.ChannelSetup.getUsedChannels();
                Col = {Chan.Color};
                Col = cell2mat(Col(:));

                obj.TimeData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    CQuantity.Voltage.textID(), CQuantity.Voltage.EU, Col);
                obj.TimeDataView.UpdateChannels();
                obj.FreqData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Frequency.textID(), CQuantity.Frequency.EU,...
                    CQuantity.Voltage.textID(), CQuantity.Voltage.EU, Col);
                obj.FreqDataView.UpdateChannels();
                
                obj.chSetupChanged = false;
            end
            
            obj.TimeDataView.unblock();
            obj.FreqDataView.unblock();
            
            % Initialize hardware for calibration
            status = obj.Hardware.MonitorInit(obj.ChannelSetup.InputChannels);
        end
        
        function hide(obj)
            %HIDE function called each time before the module is closed
            
            % block graphs for better performance while working in other modules
            obj.TimeDataView.block();
            obj.FreqDataView.block();
            
            % close hardware
            status = obj.Hardware.MonitorClose();
        end
    end
    
    methods (Access = private)
        function InitFigure(obj)
            %INITFIGURE initialize and arrange GUI objects on figure
            
            h3 = CLevelMeterView(obj.hContainer, obj.LevelMeter, 100);
            butAcq = uicontrol('parent', obj.hContainer, 'Style', 'PushButton',...
                'String', CBabel.getInstance().getText('START_CALIBRATION'), ...
                'Callback', @(src, event)CalibCallback(obj, src));
            h2 = CHardwareView(obj.hContainer, obj.Hardware);
            
            obj.FreqDataView.setScale('xy', 'log');
            
            Lmenu = {{obj.AcqSetupView.hPanel, [-1 0]};
                {obj.CalSelectView.hPanel, [-1 0]};
                {butAcq, [-1 0]};
                {h3.hPanel, [-1 -1]}
                {h2.hPanel, [-1 0]}};
            w = get(obj.AcqSetupView.hPanel, 'position');
            w = w(3);
            Lcontent = {{obj.CalPanel.hPanel, [0 0]}; {obj.TimeDataView.hPanel, [-1 -1]}; {obj.FreqDataView.hPanel, [-1 -1]}};
            Layout = {{Lmenu, [w -1]}, {Lcontent, [-1 -1]}};
            GuiLayout.ApplyLayout(obj.hContainer, Layout);
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
        end
    end
end
