classdef CChannelSetup < CUiTab
    %CCHANNELSETUP Nihu.Lab module for channel setup
    
    properties (Constant)
        ChannelColors = [
            0 0 1 % blue
            0 .5 0 % green
            1 0 0 % red
            0 .75 .75 % cyan
            .75 0 .75 % magenta
            .75 .75 0 % shit
            .25 .25 .25 % darkgrey
            0 0 0 % black
            ];
    end
    
    properties
        hOutputTables   % handles of output module tables
        
        InputModules    % vector of input modules
        InputChannels   % vector of input channels
        
        OutputModules   % vector of output modules
        OutputChannels  % vector of output channels
        
        AmpDB       % amplifiers database
        SensorDB    % sensors database
        
        ChannelViews % array of ChannelView handles
    end
    
    events
        channelSetupChange
    end
    
    methods
        function Change(obj)
            %CHANGE Called when Channel Setup has been changed
            
            notify(obj, 'channelSetupChange');
        end
        
        function show(obj)
            %SHOW function called when the module is switched to
            
            for iCh = 1 : length(obj.ChannelViews)
                obj.ChannelViews(iCh).refresh();
            end
        end
        
        function hide(obj)
            %HIDE function called when another module is switched to
            Check(obj);
        end
        
        function obj = CChannelSetup(ParentGroup, inputModules, outputModules)
            %CCHANNELSETUP constructor
            obj = obj@CUiTab(ParentGroup, 'CHANNELSETUP_MODULE');
            
            obj.InputModules = inputModules;
            obj.OutputModules = outputModules;
            
            obj.ImportDB();         % import databases
            obj.CreateChannels();   % build channel lists
            obj.InitFigure();       % initialise the gui figure
        end % of constructor
        
        function SaveCallback(obj)
            %SAVECALLBACK Called when the user presses the Save icon
            InputChannels = obj.InputChannels; %#ok<PROP,NASGU>
            OutputChannels = obj.OutputChannels; %#ok<PROP,NASGU>
            uisave({'InputChannels', 'OutputChannels'});
        end % of function SaveCallback
        
        function LoadCallback(obj)
            %LOADCALLBACK Called when the user presses the Load icon
            uiload;
            if exist('InputChannels', 'var') && exist('OutputChannels', 'var')
                for iCh = 1 : min(length(InputChannels), length(obj.InputChannels)) %#ok<CPROP>
                    obj.InputChannels(iCh).isSelected = InputChannels(iCh).isSelected; %#ok<CPROP>
                    obj.InputChannels(iCh).isReference = InputChannels(iCh).isReference; %#ok<CPROP>
                    obj.InputChannels(iCh).Name = InputChannels(iCh).Name; %#ok<CPROP>
                    obj.InputChannels(iCh).Dir = InputChannels(iCh).Dir; %#ok<CPROP>
                    obj.InputChannels(iCh).Sensor = InputChannels(iCh).Sensor; %#ok<CPROP>
                    obj.InputChannels(iCh).AmpCh = InputChannels(iCh).AmpCh; %#ok<CPROP>
                    obj.InputChannels(iCh).Color = InputChannels(iCh).Color; %#ok<CPROP>
                end
                for iCh = 1 : min(length(OutputChannels), length(obj.OutputChannels)) %#ok<CPROP>
                    obj.OutputChannels(iCh).isSelected = OutputChannels(iCh).isSelected; %#ok<CPROP>
                    obj.OutputChannels(iCh).isReference = OutputChannels(iCh).isReference; %#ok<CPROP>
                    obj.OutputChannels(iCh).Name = OutputChannels(iCh).Name; %#ok<CPROP>
                    obj.OutputChannels(iCh).Dir = OutputChannels(iCh).Dir; %#ok<CPROP>
                    obj.OutputChannels(iCh).Sensor = OutputChannels(iCh).Sensor; %#ok<CPROP>
                    obj.OutputChannels(iCh).AmpCh = OutputChannels(iCh).AmpCh; %#ok<CPROP>
                    obj.OutputChannels(iCh).Color = OutputChannels(iCh).Color; %#ok<CPROP>
                end
            else
                str = CBabel.getInstance().getText('CHSETUP_IMPORT_ERROR_NOSTRUCT');
                errordlg(str, CBabel.getInstance().getText('IMPORT_ERROR'), 'modal');
            end
        end % of function LoadCallback
        
        
        function eu = voltage2eu(obj, voltage)
            %VOLTAGE2EU convert voltage to engineering unit (Sens * Amp)
            
            Ch = obj.getUsedChannels();
            m = size(voltage);
            eu = reshape(reshape(voltage, [], m(end)) * diag(Ch.voltage2eu()), m);
        end % of function voltage2eu
        
        function y = decompensate9234(obj, y, fs)
            %DECOMPENSATE9234 Decompensates delay of 9234 devices
            
            Ch = obj.getUsedChannels();
            is9234 = ismember(Ch.getModule(), 'NI9234');
            delay = round(38.4+3.2e-6*fs); % delay from catalogue
            y(:,:,is9234) = circshift(y(:,:,is9234), [-delay 0 0]);
        end % of function decompensate9243
        
        function y = decompensate_preamp(obj, y)
            %DECOMPENSATE_PREAMP decompensate preamplification on input data
            
            Ch = obj.getUsedChannels();
            y = y * diag(1./Ch.PreAmplification());
        end % of function decompensate_preamp
        
        function n = getActiveChNum(obj)
            %GETACTIVECHNUM Return number of active channels
            
            n = length(find(cell2mat({obj.InputChannels.isSelected})));
        end
        
        function i = getActiveReferenceIndex(obj)
            %GETACTIVEREFERENCEINDEX return index of active reference ch
            Ch = obj.getUsedChannels();
            i = find(cell2mat({Ch.isReference}));
        end
        
        function [iCh, oCh] = getUsedChannels(obj)
            %GETUSEDCHANNELS return used input and output channels
            iCh = obj.InputChannels(cell2mat({obj.InputChannels.isSelected}));
            oCh = obj.OutputChannels(cell2mat({obj.OutputChannels.isSelected}));
        end
    end % of public methods
    
    methods (Access = private)
        function CreateChannels(obj)
            
            % build input channels list
            obj.InputChannels = CChannel.empty(1,0);
            nMod = length(obj.InputModules);
            nCol = size(CChannelSetup.ChannelColors,1);
            iCh = 1;
            for iMod = 1 : nMod
                nCh = obj.InputModules(iMod).nChannels;
                for iIn = 1 : nCh
                    sel = iMod == 1 && iIn == 1; % first module first is selected
                    inp = CInput(obj.InputModules(iMod), iIn, COperationMode.AC);
                    sens = obj.SensorDB(1);
                    amp = CAmpChannel(obj.AmpDB(1), 1, 1);
                    obj.InputChannels(iCh) = CChannel(iCh, sel, false,...
                        sprintf(CBabel.getInstance().getText('INPUT_CHANNEL_1'), iCh),...
                        CDirection.Scalar, inp, sens, amp,...
                        CChannelSetup.ChannelColors(mod(iCh-1, nCol)+1,:));
                    iCh = iCh + 1;
                end
            end
            
            % build output channels list
            obj.OutputChannels = CChannel.empty(1,0);
            nMod = length(obj.OutputModules);
            iCh = 1;
            for iMod = 1 : nMod
                nCh = obj.OutputModules(iMod).nChannels;
                for iIn = 1 : nCh
                    sel = iMod == 1 && iIn == 1;
                    inp = CInput(obj.OutputModules(iMod), iIn, COperationMode.AC);
                    sens = obj.SensorDB(1);
                    amp = CAmpChannel(obj.AmpDB(1), 1, 1);
                    obj.OutputChannels(iCh) = CChannel(iCh, sel, false,...
                        sprintf(CBabel.getInstance().getText('OUTPUT_CHANNEL_1'), iCh),...
                        CDirection.Scalar, inp, sens, amp,...
                        CChannelSetup.ChannelColors(mod(iCh-1, nCol)+1,:));
                    iCh = iCh + 1;
                end
            end
        end % of function CreateChannels
        
        function ImportDB(obj)
            %IMPORTDB  Import Sensor and Amplifier database
            
            name = fullfile(fileparts(mfilename('fullpath')), 'DataBase', 'Sensors.csv');
            fid = fopen(name);
            if fid == -1
                error('NiHuLab:File', CBabel.getInstance().getText('FILE_OPEN_ERROR'), name);
            end
            Sensors = textscan(fid, '%s', 'delimiter', ',');
            fclose(fid);
            Sensors = Sensors{1};
            nFields = 7;
            Sensors = reshape(Sensors(nFields+1:end), nFields, [])';
            Sensors(:,4) = num2cell(str2num(char(Sensors(:,4)))); %#ok<*ST2NM>
            Sensors(:,5) = num2cell(str2num(char(Sensors(:,5))));
            Sensors(:,7) = num2cell(logical(str2num(char(Sensors(:,7)))));
            nSens = size(Sensors,1);
            obj.SensorDB = CSensor.empty(0,1);
            for i = 1 : nSens
                sens = Sensors(i,:);
                obj.SensorDB(i,1) = CSensor(sens{2}, sens{3}, sens{4},...
                    sens{5}, CQuantity.fromtext(sens{1}), sens{7});
            end
            
            name = fullfile(fileparts(mfilename('fullpath')), 'DataBase', 'Amplifiers.csv');
            fid = fopen(name);
            if fid == -1
                error('NiHuLab:File', CBabel.getInstance().getText('FILE_OPEN_ERROR'), name);
            end
            Amps = textscan(fid, '%s', 'delimiter', ',');
            fclose(fid);
            Amps = Amps{1};
            nFields = 5;
            Amps = reshape(Amps(nFields+1:end), nFields, [])';
            Amps(:,3) = num2cell(str2num(char(Amps(:,3))));
            Amps(:,4) = num2cell(logical(str2num(char(Amps(:,4)))));
            Amps(:,5) = num2cell(str2num(char(Amps(:,5))));
            nAmp = size(Amps,1);
            obj.AmpDB = CAmplifier.None;
            for i = 1 : nAmp
                amp = Amps(i,:);
                % TODO constant [1 10 100] should be loaded from file
                obj.AmpDB(i+1,1) = CAmplifier(amp{1}, amp{2}, amp{3},...
                    amp{5}, amp{4}, [1 10 100]);
            end
        end % of function ImportDB
        
        function InitFigure(obj)
            % Save button
            icondir = fullfile(matlabroot, 'toolbox', 'matlab', 'icons');
            im = importdata(fullfile(icondir, 'file_save.png'));
            b3 = uicontrol('Parent', obj.hContainer, 'Style', 'PushButton', ...
                'String', '', 'Callback', @(src,event)SaveCallback(obj), ...
                'CData', double(im.cdata)/2^16, ...
                'ToolTipStr', CBabel.getInstance().getText('SAVE_CHSETUP'), ...
                'Units', 'Pixels',...
                'Position', [0, 0, CSetupClass.itemHeight, CSetupClass.itemHeight]);
            % Load button
            im = importdata(fullfile(icondir, 'file_open.png'));
            b4 = uicontrol('Parent', obj.hContainer, 'Style', 'PushButton', ...
                'String', '', 'Callback', @(src,event)LoadCallback(obj), ...
                'CData', double(im.cdata)/2^16, ...
                'ToolTipStr', CBabel.getInstance().getText('OPEN_CHSETUP'), ...
                'Units', 'Pixels',...
                'Position', [0, 0, CSetupClass.itemHeight, CSetupClass.itemHeight]);
            
            nMod = length(obj.InputModules);
            NCh = 0;
            for iMod = 1 : nMod
                NCh = NCh + obj.InputModules(iMod).nChannels;
            end
            
            obj.ChannelViews = CChannelView.empty();
            
            hInPan = zeros(1,nMod);
            kCh = 0;
            for iMod = 1 : nMod
                nCh = obj.InputModules(iMod).nChannels;
                P = CPanel(obj.hContainer, obj.InputModules(iMod).Name);
                hInPan(iMod) = P.hPanel;
                
                lp = cell(nCh,1);
                for iCh = 1 : nCh
                    kCh = kCh + 1;
                    
                    obj.ChannelViews(kCh) = CChannelView(obj.InputChannels(kCh), obj, P.hPanel);
                    lp{iCh}{1} = obj.ChannelViews(kCh).layout;
                    lp{iCh}{2} = [-1 0];
                end
                GuiLayout.ApplyLayout(P.hPanel, lp);
            end
            
            li = repmat({{0, [-1 0]}}, nMod,1);
            for n = 1 : nMod
                li{n}{1}(1) = hInPan(n);
            end
            
            l2 = {{b3, [0 0]},{b4, [0 0]}};
            l = {{li, [-1 0]}; {l2, [0 0]}};
            GuiLayout.ApplyLayout(obj.hContainer, l);
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
            
            GuiLayout.ApplyLayout(obj.hContainer, l);
        end % of function InitFigure2
        
        function ok = Check(obj)
            % Select selected channels, report error if none
            if obj.getActiveChNum() == 0
                errordlg(CBabel.getInstance().getText('NO_CHSEL_ERROR'), CBabel.getInstance().getText('CHSETUP_ERROR'));
                ok = false;
                return
            end
            
            % Check ICP inconsistency
            for iCh = 1 : length(obj.InputChannels)
                sens = obj.InputChannels(iCh).Sensor;
                amp = obj.InputChannels(iCh).AmpCh.Amplifier;
                mod = obj.InputChannels(iCh).Inp.Mod;
                if ~mod.provideICP && ~amp.provideICP && sens.requireICP
                    str = sprintf(CBabel.getInstance().getText('CHSETUP_ICP_ERROR'),...
                        mod.Name, ...
                        physNr(iCh), ...
                        obj.InputChannels(physNr(iCh)).Name, ...
                        sens.Name);
                    errordlg(str, CBabel.getInstance().getText('CHSETUP_ERROR'));
                    ok = false;
                    return
                end
            end
            
            % Check Preamp coincidence
            %             amps = setdiff(D(:,10), 'None');
            %             if length(amps) > length(unique(amps))
            %                 errordlg(CBabel.getInstance().getText('PREAMP_COLLISION_ERROR'), CBabel.getInstance().getText('CHSETUP_ERROR'));
            %                 ok = false;
            %                 return
            %             end
            
            ok = true;
        end % of function Check
    end % of private methods
end % of class CChannelSetup
