classdef CChannelView < CGuiView
    %CCHANNELVIEW View class of CChannel
    
    properties (SetAccess = immutable)
        SensorDB    % handle to the sensor database
        AmplifierDB % handle to the amplifier database
        ChSetup     % handle of the channel setup
    end
    
    properties
        hSelected       % handle of the selection uicontrol
        hReference      % handle of the reference selection uicontrol
        hName           % handle of the name selection uicontrol
        hQuantity       % handle of the quantity selection uicontrol
        hDirection      % handle of the direction selection uicontrol
        hSensor         % handle of the sensor selection uicontrol
        hSensitivity    % handle of the sensitivity display uicontrol
        hAmplifier      % handle of the amplifier selection uicontrol
        hAmplifierCh    % handle of amplifier channel selection uicontrol
        hAmplification  % handle of amplification selection uicontrol
        hColor          % handle of the color selection uicontrol
        
        layout
    end
    
    methods
        function obj = CChannelView(hData, ChSetup, hParent)
            %CCHANELVIEW Constructor
            obj = obj@CGuiView(hData, {...
                'isSelected',...
                'isReference',...
                'Name',...
                'Dir',...
                'Sensor',...
                'AmpCh', ...
                'Color'});
            obj.ChSetup = ChSetup;
            obj.SensorDB = obj.ChSetup.SensorDB;
            obj.AmplifierDB = obj.ChSetup.AmpDB;
            obj.InitObject(hParent);
            obj.refresh();
        end
        
        function refresh(obj)
            %REFRESH function called when underlying data object is changed
            set(obj.hSelected, 'value', obj.hData.isSelected);
            set(obj.hReference, 'value', obj.hData.isReference);
            set(obj.hName, 'string', obj.hData.Name);
            % adjust quantity
            quants = get(obj.hQuantity, 'string');
            quant = obj.hData.Sensor.Quantity;
            v = find(strcmp(quant.char(), quants), 1, 'first');
            set(obj.hQuantity, 'value', v);
            % adjust direction
            if quant.isScalar
                dirs = {
                    CDirection.Scalar.char();
                    };
            else
                dirs = {
                    CDirection.pX.char()
                    CDirection.mX.char()
                    CDirection.pY.char()
                    CDirection.mY.char()
                    CDirection.pZ.char()
                    CDirection.mZ.char()
                    };
            end
            dir = obj.hData.Dir;
            v = find(strcmp(dir.char(), dirs), 1, 'first');
            set(obj.hDirection, 'string', dirs, 'value', v);
            % adjust sensors
            sel = false(length(obj.SensorDB),1);
            for i = 1 : length(obj.SensorDB)
                sel(i) = quant == obj.SensorDB(i).Quantity;
            end
            str = {obj.SensorDB(sel).Name};
            v = find(strcmp(obj.hData.Sensor.Name, str));
            set(obj.hSensor, 'string', str, 'value', v);
            set(obj.hSensitivity, 'string', sprintf('%g V/%s (%g V/%s)',...
                obj.hData.Sensor.NominalSensitivity, obj.hData.Sensor.Quantity.EU, ...
                obj.hData.Sensor.ActualSensitivity, obj.hData.Sensor.Quantity.EU));
            % adjust amplifiers
            amps = get(obj.hAmplifier, 'string');
            amp = obj.hData.AmpCh.Amplifier;
            ampf = amp.Amplifications(:);
            v = find(strcmp(amp.Name(), amps));
            set(obj.hAmplifier, 'value', v);
            set(obj.hAmplifierCh,...
                'string', cellstr(num2str((1:amp.NumChannels)')), ...
                'value', obj.hData.AmpCh.ChNum);
            set(obj.hAmplification,...
                'string', cellstr(num2str(ampf)), ...
                'value', find(obj.hData.AmpCh.Amplification==ampf, 1, 'first'));
            % adjust color
            set(obj.hColor, 'backgroundcolor', obj.hData.Color, ...
                'string', num2str(obj.hData.PhysChNr), ...
                'ForegroundColor', 'white', ...
                'FontWeight', 'bold');
            
            % adjust enable status
            if obj.hData.isSelected
                set([obj.hReference, obj.hName,...
                    obj.hQuantity, obj.hDirection, obj.hSensor, ...
                    obj.hSensitivity, obj.hAmplifier, obj.hAmplifierCh, ...
                    obj.hAmplification, obj.hColor], 'enable', 'on');
                if amp == CAmplifier.None
                    set([obj.hAmplifierCh, obj.hAmplification], 'enable', 'off');
                end
            else
                set([obj.hReference, obj.hName,...
                    obj.hQuantity, obj.hDirection, obj.hSensor, ...
                    obj.hSensitivity, obj.hAmplifier, obj.hAmplifierCh, ...
                    obj.hAmplification, obj.hColor], 'enable', 'off');
            end
        end
        
        function SelectCallback(obj)
            %SELECTCALLBACK Called when selected chkbox is changed

            v = get(obj.hSelected, 'value');
            obj.hData.isSelected = logical(v);
            obj.ChSetup.Change();
        end
        
        function ReferenceCallback(obj)
            %REFERENCECALLBACK Called when reference chkbox is changed

            v = get(obj.hReference, 'value');
            obj.hData.isReference = logical(v);
            obj.ChSetup.Change();
        end
        
        function NameCallback(obj)
            %NAMECALLBACK Called when name edit is changed

            v = get(obj.hName, 'string');
            v = strtrim(v);
            if ~isempty(v)
                obj.hData.Name = v;
            end
            obj.ChSetup.Change();
        end
        
        function DirectionCallback(obj)
            %DIRECTIONCALLBACK Called when direction popup is changed

            v = get(obj.hDirection, 'value');
            str = get(obj.hDirection, 'string');
            obj.hData.Dir = CDirection.fromtext(str{v});
            obj.ChSetup.Change();
        end
        
        function QuantityCallback(obj)
            %QUANTITYCALLBACK Called when quantity popup is changed
            
            % retrieve current quantity from popup menu
            v = get(obj.hQuantity, 'value');
            str = get(obj.hQuantity, 'string');
            q = CQuantity.fromtext(str{v});
            
            % set direction according to quantity
            if q.isScalar && obj.hData.Dir ~= CDirection.Scalar
                obj.hData.Dir = CDirection.Scalar;
            elseif ~q.isScalar && obj.hData.Dir == CDirection.Scalar
                obj.hData.Dir = CDirection.pX;
            end
            
            % set default sensor to new quantity
            for i = 1 : length(obj.SensorDB)
                if q == obj.SensorDB(i).Quantity
                    obj.hData.Sensor = obj.SensorDB(i);
                    obj.ChSetup.Change();
                    break;
                end
            end
        end
        
        function SensorCallback(obj)
            %AMPLIFIERCALLBACK Called when sensor popup is changed

            v = get(obj.hSensor, 'value');
            str = get(obj.hSensor, 'string');
            s = str{v};
            for i = 1 : length(obj.SensorDB)
                if strcmp(s, obj.SensorDB(i).Name())
                    obj.hData.Sensor = obj.SensorDB(i);
                    obj.ChSetup.Change();
                    break;
                end
            end
        end
        
        function AmplifierCallback(obj)
            %AMPLIFIERCALLBACK Called when amplifier popup is changed

            v = get(obj.hAmplifier, 'value');
            str = get(obj.hAmplifier, 'string');
            a = str{v};
            for i = 1 : length(obj.AmplifierDB)
                if strcmp(a, obj.AmplifierDB(i).Name())
                    obj.hData.AmpCh = CAmpChannel(obj.AmplifierDB(i));
                    obj.ChSetup.Change();
                    break;
                end
            end
            obj.ChSetup.Change();
        end
        
        function AmplifierChCallback(obj)
            %AMPLIFIERCHCALLBACK Called when amplifier channel popup is changed

            v = get(obj.hAmplifierCh, 'value');
            str = get(obj.hAmplifierCh, 'string');
            obj.hData.AmpCh.ChNum = str2double(str{v});
            obj.ChSetup.Change();
        end
        
        function AmplificationCallback(obj)
            %AMPLIFICATIONCALLBACK Called when amplification popup is changed

            v = get(obj.hAmplification, 'value');
            str = get(obj.hAmplification, 'string');
            obj.hData.AmpCh.Amplification = str2double(str{v});
            obj.ChSetup.Change();
        end
        
        function ColorCallback(obj)
            %COLORCALLBACK Called when Channel Color selection is changed

            obj.hData.Color = uisetcolor(obj.hData.Color);
            obj.ChSetup.Change();
        end
    end % of public methods
    
    methods (Access = private)
        function InitObject(obj, hParent)
            % select direction values
            [~, Dirs] = enumeration('CDirection');
            
            % select quantities from the sensor database
            Quant = cell(length(obj.SensorDB),1);
            for i = 1 : length(obj.SensorDB)
                Quant{i} = obj.SensorDB(i).Quantity.char();
            end
            Quant = unique(Quant);
            
            obj.hSelected = uicontrol(...
                'parent', hParent, ...
                'style', 'checkbox', ...
                'callback', @(src,evt)SelectCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_SELECT_TOOLTIP'));
            w = get(obj.hSelected, 'position');
            w = w(4);
            set(obj.hSelected, 'position', [0 0 w w]);
            obj.hReference = uicontrol(...
                'parent', hParent, ...
                'style', 'checkbox', ...
                'callback', @(src,evt)ReferenceCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_REFERENCE_TOOLTIP'));
            set(obj.hReference, 'position', [0 0 w w]);
            obj.hName = uicontrol(...
                'parent', hParent, ...
                'style', 'edit', ...
                'horizontalalignment', 'left', ...
                'backgroundcolor', 'white', ...
                'callback', @(src,evt)NameCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_NAME_TOOLTIP'));
            obj.hQuantity = uicontrol(...
                'parent', hParent, ...
                'style', 'popupmenu', ...
                'backgroundcolor', 'white', ...
                'string', Quant, ...
                'callback', @(src,evt)QuantityCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_QUANTITY_TOOLTIP'));
            obj.hDirection = uicontrol(...
                'parent', hParent, ...
                'style', 'popupmenu', ...
                'backgroundcolor', 'white', ...
                'string', Dirs, ...
                'callback', @(src,evt)DirectionCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_DIRECTION_TOOLTIP'));
            obj.hSensor = uicontrol(...
                'parent', hParent, ...
                'style', 'popupmenu', ...
                'backgroundcolor', 'white', ...
                'string', 'invalid', ...
                'callback', @(src,evt)SensorCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_SENSOR_TOOLTIP'));
            obj.hSensitivity = uicontrol(...
                'parent', hParent, ...
                'style', 'text', ...
                'string', '', ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_SENSITIVITY_TOOLTIP'));
            obj.hAmplifier = uicontrol(...
                'parent', hParent, ...
                'style', 'popupmenu', ...
                'backgroundcolor', 'white', ...
                'string', {obj.AmplifierDB.Name}, ...
                'callback', @(src,evt)AmplifierCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_AMPLIFIER_TOOLTIP'));
            obj.hAmplifierCh = uicontrol(...
                'parent', hParent, ...
                'style', 'popupmenu', ...
                'backgroundcolor', 'white', ...
                'string', 'invalid', ...
                'callback', @(src,evt)AmplifierChCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_AMPLIFIER_CHANNEL_TOOLTIP'));
            obj.hAmplification = uicontrol(...
                'parent', hParent, ...
                'style', 'popupmenu', ...
                'backgroundcolor', 'white', ...
                'string', 'invalid', ...
                'callback', @(src,evt)AmplificationCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_AMPLIFICATION_TOOLTIP'));
            obj.hColor = uicontrol(...
                'parent', hParent, ...
                'style', 'pushbutton', ...
                'string', '', ...
                'position', [0 0 w w], ...
                'callback', @(src,evt)ColorCallback(obj), ...
                'tooltip', CBabel.getInstance().getText('CHANNEL_COLOR_TOOLTIP'));
            
            obj.layout = {
                {obj.hColor, [0 0]}, ...
                {obj.hSelected, [0 0]}, ...
                {obj.hReference, [0 0]}, ...
                {obj.hName, [-3 0]}, ...
                {obj.hQuantity, [-2 0]}, ...
                {obj.hDirection, [-1 0]}, ...
                {obj.hSensor, [-3 0]}, ...
                {obj.hSensitivity, [-2 0]}, ...
                {obj.hAmplifier, [-3 0]}, ...
                {obj.hAmplifierCh, [-1 0]}, ...
                {obj.hAmplification, [-1 0]}
                };
        end
    end % of private methods
end % of classdef
