%CDISTORTION Nihu.Lab module for Distortion Measurement
classdef CDistortion < CUiTab & CChSetupSensitive
    properties
        Hardware        % handle to CHardware object
        AcqSetup        % handle to CAcqSetup object
        AcqSetupView    % handle to CAcqSetupView object
        DistSignal      % handle to CDistSignal object
        DistSignalView  % handle to CDistSignalView object
        LevelMeter      % handle to CLevelMeter object
        FileSelect      % handle to CFileSelect object
        FileSelectView  % handle to CFileSelectView object
        
        MeasurementResult   % measurement results are stored in this structure
        
        ExcData         % CGraphData object (excitation time history)
        ExcDataView     % handle to CGraphDataView object
        RespData        % CGraphData object (response time history)
        RespDataView    % handle to CGraphDataView object
        
        cancel          % indicates if measurement is cancelled
    end
    
    methods
        function obj = CDistortion(ParentGroup, hardware, chsetup, acqsetup,...
                distsignal, levmeter)
            obj = obj@CUiTab(ParentGroup, 'DISTORTION_MODULE');
            obj = obj@CChSetupSensitive(chsetup);
            
            obj.Hardware = hardware;
            obj.AcqSetup = acqsetup;
            obj.DistSignal = distsignal;
            obj.LevelMeter = levmeter;
            
            obj.FileSelect = CFileSelect({'mat'});
            
            obj.ExcData = CGraphData();
            obj.RespData = CGraphData();
            
            obj.InitFigure();
        end
        
        function SaveCallback(obj, ~)
            %SAVECALLBACK called when the save button is pressed
            
            Result = obj.MeasurementResult; %#ok<NASGU>
            fname = fullfile(obj.FileSelect.getFullFileName());
            save(fname, 'Result');
        end
        
        function cancelMeasure(obj)
            %CANCELMEASURE called when the cancel button is pressed
            
            obj.cancel = true;
        end
        
        function DistortionCallback(obj, hButton)
            %DISTPRTIONCALLBACK called when the start calibration is pressed
            
            % read acquisition setup information
            Fs = obj.AcqSetup.Fs;
            nSamples = obj.AcqSetup.nSamples;
            
            % set averaging parameters
            nPre = 1;
            nAverage = 1;
            nPost = 1;
            
            % Block user interfaces
            set(gcbf, 'UserData', get(gcbf, 'CloseRequestFcn'));
            set(gcbf, 'CloseRequestFcn', '');
            obj.hParentGroup.block();           % cannot switch to other module
            obj.AcqSetupView.block();           % cannot change acq setup
            obj.DistSignalView.block();         % cannot change excitation
            set(hButton, 'Enable', 'off');      % cannot push acq button
            
            try
                % Generate excitation
                [Amp, Freqs, Nvec] = obj.DistSignal.FreqsN(Fs, nSamples);
                nFreq = length(Freqs);
                obj.cancel = false;
                wb = waitbar(0/nFreq, '', 'CreateCancelBtn', @(src,data)cancelMeasure(obj));
                x = cell(nFreq,1);
                y = cell(nFreq,1);
                for iFreq = 1 : nFreq
                    if obj.cancel
                        break;
                    end
                    nSamp = Nvec(iFreq);
                    freq = Freqs(iFreq);
                    tScale = (0 : nSamp-1)' / Fs;
                    x{iFreq} = Amp * sin(2*pi*freq*tScale);
                    [~, y_local] = obj.Hardware.TransferAcq(nPre, nAverage, nPost, obj.AcqSetup.Fs, x{iFreq});
                    
                    % take channel sensitiviy into account
                    y_local = obj.ChannelSetup.voltage2eu(y_local);
                    
                    y_local = obj.ChannelSetup.decompensate9234(y_local, Fs);
                    
                    refind = obj.ChannelSetup.getActiveReferenceIndex();
                    if ~isempty(refind)
                        x{iFreq} = squeeze(y_local(:,:,refind));
                    end
                    
                    obj.LevelMeter.UpdateLevels(squeeze(max(y_local,[],2)));
                    % because only one average is taken for sure
                    y{iFreq} = squeeze(y_local);
                    
                    waitbar(iFreq/nFreq, wb, sprintf('processing frequency %g', freq));
                end
                set(wb, 'CloseRequestFcn', 'closereq');
                close(wb);
                
                if ~obj.cancel
                    obj.ExcData.UpdateData(tScale, squeeze(x{end}));
                    obj.RespData.UpdateData(tScale, squeeze(y{end}));
                    
                    % save settings and results into member structure
                    [obj.MeasurementResult.InputChannels, ...
                        obj.MeasurementResult.OutputChannels] =...
                        obj.ChannelSetup.getUsedChannels();
                    obj.MeasurementResult.AcqSetup = obj.AcqSetup.dcopy();
                    obj.MeasurementResult.Frequencies = Freqs;
                    obj.MeasurementResult.ExcitationSignal = x;
                    obj.MeasurementResult.ResponseSignal = y;
                end
            catch ME
                errordlg(sprintf('%s\n%s\n%s:%d', ME.identifier, ME.message, ME.stack(1).name, ME.stack(1).line), '!!! Error !!!', 'modal');
            end
            
            % Unblock User Interfaces
            set(gcbf, 'CloseRequestFcn', get(gcbf, 'UserData'));
            obj.hParentGroup.unblock();
            obj.AcqSetupView.unblock();
            obj.DistSignalView.unblock();
            
            set(hButton, 'Enable', 'on');
        end % of function DistortionCallback
    end % of public methods
    
    methods (Access = private)
        function InitFigure(obj)
            obj.AcqSetupView = CAcqSetupView(obj.hContainer, obj.AcqSetup);
            obj.DistSignalView = CDistSignalView(obj.hContainer, obj.DistSignal);
            h2 = CLevelMeterView(obj.hContainer, obj.LevelMeter, 100);
            obj.FileSelectView = CFileSelectView(obj.hContainer, obj.FileSelect);
            butAcq = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton',...
                'Tag', 'START_DISTORTION',...
                'String', CBabel.getInstance().getText('START_DISTORTION'), ...
                'Callback', @(src, event)DistortionCallback(obj, src));
            butSav = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton',...
                'Tag', 'SAVE_DISTORTION',...
                'String', CBabel.getInstance().getText('SAVE_DISTORTION'), ...
                'Callback', @(src, event)SaveCallback(obj, src));
            h3 = CHardwareView(obj.hContainer, obj.Hardware);
            obj.ExcDataView = CGraphDataView(obj.hContainer, obj.ExcData, 'EXCITATION_TIME_HISTORY');
            obj.RespDataView = CGraphDataView(obj.hContainer, obj.RespData, 'RESPONSE_TIME_HISTORY');
            w = get(obj.AcqSetupView.hPanel, 'position');
            w = w(3);
            lMenu = {{obj.AcqSetupView.hPanel, [-1 0]};
                {obj.DistSignalView.hPanel, [-1 0]};
                {obj.FileSelectView.hPanel, [-1 0]};
                {butSav, [-1 CSetupClass.itemHeight]};
                {butAcq, [-1 CSetupClass.itemHeight]};
                {h2.hPanel, [-1 -1]};
                {h3.hPanel, [-1 0]}};
            content = {{
                {
                {obj.ExcDataView.hPanel, [-1 -1]};
                {obj.RespDataView.hPanel, [-1 -1]}
                }, [-1 -1]}};
            Layout = {{lMenu, [w, -1]} {content, [-1 -1]}};
            GuiLayout.ApplyLayout(obj.hContainer, Layout);
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
        end
    end % of private methods
    
    methods
        function show(obj)
            %SHOW Called when the user opens this GUI module
            
            obj.LevelMeter.Reset();
            
            if obj.chSetupChanged
                Chan = obj.ChannelSetup.getUsedChannels();
                Col = {Chan.Color};
                Col = cell2mat(Col(:));
                
                obj.ExcData.UpdateChannels(0,...
                    {CBabel.getInstance().getText('EXCITATION')},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    {CQuantity.Voltage.textID()}, {CQuantity.Voltage.EU}, Col(1,:));
                obj.ExcDataView.UpdateChannels();
                
                obj.RespData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                obj.RespDataView.UpdateChannels();
                
                obj.chSetupChanged = false;
            end
            
            obj.ExcDataView.unblock();
            obj.RespDataView.unblock();
            
            status = obj.Hardware.TransferInit(obj.ChannelSetup.InputChannels, obj.ChannelSetup.OutputChannels);
        end
        
        function hide(obj)
            %HIDE Called when the user switches to an other GUI module
            
            obj.ExcDataView.block();
            obj.RespDataView.block();
            
            status = obj.Hardware.TransferClose();
        end
    end % of public methods
end % of classdef
