classdef CFPGA < CNi9234Hardware
    %CFPGA Hardware class to provide interface to NI CRIO FPGA
    %   This class implements an interface between NiHu.Lab and NI CRIO
    %   FPGA data acquisition hardware.
    
    properties (Constant = true)
        DMADepth = 2e6;         % Depth of input and output DMA on HOST side
        TriggerBlockDuration = .1; % Software trigger block size in sec
    end
    
    properties (SetObservable = true)
        nSamplesInBuffer    % number of samples in buffer
    end
    
    properties
        InputModules             % vector of NIFPGA CModule objects
        OutputModules             % vector of NIFPGA CModule objects
        nInput              % number of input modules
        nCh         % number of channels
        
        nPretrigSamples     % number of PreTrigger Samples
        nPosttrigSamples    % number of PostTrigger Samples
        nSamples            % total number of samples to acquire
        
        ImpactBuffer        % internal Buffer for impact testing hw trigger
        ImpactBufferIndex   % index pointer of circular buffer
        
        CRIO        % FPGA MEX object
        acquireFPGA % Measurement VI object
        Context     % For interrupt handling
        
        inputLSB         % LSB of channels
        inputOffset      % Offset of channels
        nICPInput   % number of ICP inputs
        
        outputLSB         % LSB of output channels
        outputOffset      % Offset of output channels
        ChSel       % Channel Selection logical array
    end
    
    methods
        % Constructor
        function obj = CFPGA()
            obj = obj@CNi9234Hardware();
            % Search for Target
            Targets = {
                'rio://169.254.125.17/RIO0',... % cross cable
                'rio://152.66.249.36/RIO0' % BME HIT URL
                };
            for iTarget = 1 : length(Targets)
                Target = Targets{iTarget};
                obj.CRIO = CCRIO(Target);
                % the following error usually occurs when the FPGA module
                % is not installed
                if obj.CRIO.Status < 0
                    error('NiHuLab:CFPGA:Hardware', status2msg(obj.CRIO.Status));
                end
                % try to open the FPGA module
                status = obj.CRIO.FPGAInfo.open('norun');
                switch status
                    case -63040 % RPC connection error
                        fprintf(1,'CRIO not found at %s\n', Targets{iTarget});
                        delete(obj.CRIO);
                    case 0 % CRIO has been found
                        obj.CRIO.FPGAInfo.close;
                        fprintf(1,'CRIO found at %s\n', Targets{iTarget});
                        break;
                end
            end
            % if CRIO has not been found return an error
            if status < 0
                error('NiHuLab:CFPGA:Hardware', 'CRIO not found');
            end
            
            % Read FPGA layout information (fill obj.InputModules array)
            obj.getModules();
            
            obj.nInput = length(obj.InputModules)*4;   % #input channels
            obj.nICPInput = length(find(cell2mat({obj.InputModules.provideICP})))*4; %#ICP input modules
            
            % collect channel LSB and Offset vectors
            obj.inputLSB = [];
            obj.inputOffset = [];
            for i = 1 : length(obj.InputModules)
                obj.inputLSB = [obj.inputLSB obj.InputModules(i).LSB];
                obj.inputOffset = [obj.inputOffset obj.InputModules(i).Offset];
            end
            
            obj.outputLSB = [];
            obj.outputOffset = [];
            for i = 1 : length(obj.OutputModules)
                obj.outputLSB = [obj.outputLSB obj.OutputModules(i).LSB];
                obj.outputOffset = [obj.outputOffset obj.OutputModules(i).Offset];
            end
        end
        
        function getModules(obj)
            % Return array of modules inserted into the NI hardware
            
            % Open and Run FPGAInfo Module
            status = obj.CRIO.FPGAInfo.open('norun');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(obj.CRIO.Status));
            end;
            status = obj.CRIO.FPGAInfo.run('wait');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(obj.CRIO.Status));
            end;
            
            % Read Serials
            SN = obj.CRIO.FPGAInfo.SerialNumber;
            % Close module
            status = obj.CRIO.FPGAInfo.close();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(obj.CRIO.Status));
            end;
            
            obj.InputModules = CModule.empty(0,1);
            obj.OutputModules = CModule.empty(0,1);
            usedslots = [];
            for i = 1 : length(SN)
                if SN(i) ~= 0  % serial of empty slot is 0
                    mod = CModule.FromSN(SN(i));
                    switch i
                        case {2, 4} % 9234 modules should be placed in slots 2, 4
                            if mod ~= CModule.NI9234a && mod ~= CModule.NI9234b && mod ~= CModule.NI9234c
                                continue;
                            end
                        case 6 % 9215 module should be placed in slot 6
                            if mod ~= CModule.NI9215
                                continue;
                            end
                        case 8 % 9263 module should be placed in slot 8
                            if mod ~= CModule.NI9263a && mod ~= CModule.NI9263b 
                                continue;
                            end
                    end
                    usedslots(end+1,1) = i;
                    if mod.isInput
                        obj.InputModules(end+1,1) = mod;
                    else
                        obj.OutputModules(end+1,1) = mod;
                    end
                end
            end
            try
                % Try to open a BitFile that matches the current module
                % layout
                obj.acquireFPGA = obj.CRIO.(sprintf('acquireFPGA%s', sprintf('%d', usedslots)));
            catch ME
                error('NiHuLab:CCRIO:Hardware', 'Invalid hardware configuration');
            end
        end % of function getModules()
        
        function voltage = raw2calibrated(obj, Samples)
            % convert Integer Samples to voltage
            
            lsb = obj.inputLSB(obj.ChSel);
            offset = obj.inputOffset(obj.ChSel);
            voltage = double(Samples);
            for iCh = 1 : size(Samples,2)
                voltage(:,iCh) = voltage(:,iCh) * lsb(iCh) - offset(iCh);
            end
        end
        
        function samples = calibrated2raw(obj, voltage)
            % convert Voltage to Integer Samples
            
            lsb = obj.outputLSB(end-3);
            offset = obj.outputOffset(end-3);
            samples = int16(round((voltage + offset) / lsb));
        end
        
        function status = MonitorInit(obj, Channels)
            % Initialise Monitor Module
            
            % open FPGA without running it
            status = obj.acquireFPGA.open('norun');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            % Write Channel Setup and ICP configuration into FPGA
            obj.ChSel = cell2mat({Channels.isSelected});
            obj.nCh = length(find(obj.ChSel));
            OpMode = zeros(obj.nICPInput,1);
            for i = 1 : obj.nICPInput
                OpMode(i) = Channels(i).OpMode.FPGANum;
            end
            obj.acquireFPGA.ChannelSelect = obj.ChSel;
            if isfield(obj.acquireFPGA, 'OutputEnable')
                obj.acquireFPGA.OutputEnable = false(4,1);
            end
            obj.acquireFPGA.OperationMode = OpMode;
            % Configure and start DMA
            status = obj.acquireFPGA.DMAInput.configure(obj.DMADepth);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            status = obj.acquireFPGA.DMAInput.start();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            % Reserve IRQ
            [status, obj.Context] = obj.acquireFPGA.reserveIrq();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            % Run FPGA VI
            status = obj.acquireFPGA.run('nowait');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
        end % of MonitorInit
        
        function [status, starttime]= MonitorStart(obj, Fs, ~)
            % Start Monitoring
            
            DataRate = CFPGA.FsToDataRate(Fs);
            
            % Set actual sampling frequency
            obj.acquireFPGA.DataRate = DataRate;
            % Set all stop controls to false
            obj.acquireFPGA.StopSegment = 0;
            obj.acquireFPGA.stop = 0;
            % Set acq mode to continuous
            obj.acquireFPGA.AcquisitionMode = 1; % Continuous
            % wait until IRQ channel is free
            [status, asserted] = obj.acquireFPGA.waitOnIrq(obj.Context, 15, 5000);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            % Measurement is triggered when IRQ is acknowledged
            status = obj.acquireFPGA.ackIrq(asserted);
            starttime = now(); % starttime is needed for streaming
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
        end % of function MonitorStart()
        
        function [status, voltage] = MonitorAcq(obj, nSamples)
            % Acquire samples during monitoring
            
            % acquire given number of samples
            [status, Samples, obj.nSamplesInBuffer] =...
                obj.acquireFPGA.DMAInput.read(obj.nCh*nSamples, Inf);
            % convert to voltage
            voltage = obj.raw2calibrated(reshape(Samples, obj.nCh, nSamples)');
            disp(obj.acquireFPGA.DataRate);
        end % of function MonitorAcq()
        
        function [status, rest] = MonitorStop(obj)
            % Stop Monitoring
            
            % Set Segment Stop to true. This ends FPGA program
            obj.acquireFPGA.StopSegment = 1;
            % Flush DMA
            [status, rest] = CFPGA.FlushDMA(obj.acquireFPGA.DMAInput);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            obj.nSamplesInBuffer = 0;
            % if needed, return remaining samples (needed for streaming)
            if nargout > 1
                rest = obj.raw2calibrated(reshape(rest, obj.nCh, floor(numel(rest)/obj.nCh))');
            end
        end % of function MonitorStop()
        
        function status = MonitorClose(obj)
            % Close Monitor Module
            
            % unreserve IRQ context
            if ~isempty(obj.Context)
                status = obj.acquireFPGA.unreserveIrq(obj.Context);
                obj.Context = [];
                if status < 0
                    error('NiHuLab:CFPGA:Hardware', status2msg(status));
                end
            end
            % close module
            status = obj.acquireFPGA.close();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
        end % of function MonitorClose()
        
        
        function status = TransferInit(obj, Channels, ~)
            % Initialise FPGA for Transfer module
            
            % open FPGA without running it
            status = obj.acquireFPGA.open('norun');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Write Channel Setup and ICP configuration into FPGA
            obj.ChSel = cell2mat({Channels.isSelected});
            obj.nCh = length(find(obj.ChSel));
            OpMode = zeros(obj.nICPInput,1);
            for i = 1 : obj.nICPInput
                OpMode(i) = Channels(i).OpMode.FPGANum;
            end
            obj.acquireFPGA.ChannelSelect = obj.ChSel;
            obj.acquireFPGA.OperationMode = OpMode;
            if isprop(obj.acquireFPGA, 'OutputEnable')
                obj.acquireFPGA.OutputEnable = [true false false false]';
            end
            obj.acquireFPGA.AcquisitionMode = 0; % N Samples
            
            % Configre DMA input and output
            status = obj.acquireFPGA.DMAInput.configure(obj.DMADepth);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            status = obj.acquireFPGA.DMAOutput.configure(obj.DMADepth);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Reserve IRQ Context
            [status, obj.Context] = obj.acquireFPGA.reserveIrq();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
        end % of function TranferInit()
        
        function [status y]= TransferAcq(obj, nPre, nAverage, nPost, Fs, x)
            % Acquire transfer functions
            
            DataRate = CFPGA.FsToDataRate(Fs);
            
            % Write DataRate and number of samples into FPGA
            obj.acquireFPGA.DataRate = DataRate;
            nSamples = size(x,1); %#ok<PROP>
            nTot = nPre + nAverage + nPost;
            obj.acquireFPGA.NumSamples = nTot * nSamples; %#ok<PROP>
            
            % convert outut voltage to output samples
            X = obj.calibrated2raw(x);
            
            % allocate space for input samples
            y = zeros(nSamples, nAverage, obj.nCh); %#ok<PROP>
            
            Blocks = CFPGA.BlockSize(obj.DMADepth, obj.nCh, nSamples); %#ok<PROP> % blocks
            nBlock = length(Blocks);    % number of blocks
            if nBlock > 1
                Indices = [0; cumsum(Blocks(1:end-1))];
            else
                Indices = 0;
            end
            
            % Write First Cycles to output
            nFirst = min(nTot, floor(obj.DMADepth/nSamples)); %#ok<PROP>
            if size(X,2) == 1
                XX = repmat(X, nFirst, 1);
            else
                XX = X(:,1:nFirst);
            end
            status = obj.acquireFPGA.DMAOutput.write(XX(:), Inf);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Start input and output DMA
            status = obj.acquireFPGA.DMAOutput.start();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            status = obj.acquireFPGA.DMAInput.start();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Run FPGA code (not acquisition yet)
            status = obj.acquireFPGA.run('nowait');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Run Acquisition now
            [status, asserted] = obj.acquireFPGA.waitOnIrq(obj.Context, 15, 5000);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            status = obj.acquireFPGA.ackIrq(asserted);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            for iCyc = 1 : nTot
                % read input
                for iBlock = 1 : nBlock
                    nsamp = Blocks(iBlock);
                    [status, Samples, obj.nSamplesInBuffer] =...
                        obj.acquireFPGA.DMAInput.read(obj.nCh*nsamp, Inf);
                    if status < 0
                        error('NiHuLab:CFPGA:Hardware', status2msg(status));
                    end
                    
                    % store main cycles only and discard pre and post cycle
                    % samples
                    if iCyc > nPre && iCyc <= nTot - nPost
                        y(Indices(iBlock)+(1:nsamp),iCyc-nPre,:) =...
                            obj.raw2calibrated(reshape(Samples, obj.nCh, nsamp).');
                    end
                end
                if status < 0
                    error('NiHuLab:CFPGA:Hardware', status2msg(status));
                end
                
                % write output if needed
                if iCyc <= nTot - nFirst
                    if size(x,2) == 1
                        status = obj.acquireFPGA.DMAOutput.write(X, Inf);
                    else
                        status = obj.acquireFPGA.DMAOutput.write(X(:,iCyc+nFirst), Inf);
                    end
                    if status < 0
                        error('NiHuLab:CFPGA:Hardware', status2msg(status));
                    end
                end
            end
        end % of function TransferAcq()
        
        function status = TransferClose(obj)
            % Close FPGA in Transfer Module
            
            % unreserve IRQ Context if needed
            if ~isempty(obj.Context)
                status = obj.acquireFPGA.unreserveIrq(obj.Context);
                obj.Context = [];
                if status < 0
                    error('NiHuLab:CFPGA:Hardware', status2msg(status));
                end
            end
            
            % close FPGA program
            status = obj.acquireFPGA.close();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
        end % of TransferClose
        
        function status = ImpactInit(obj, Channels)
            % Initialise FPGA for Impact module
            
            % open FPGA without running it
            status = obj.acquireFPGA.open('norun');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Write Channel Setup and ICP configuration into FPGA
            obj.ChSel = cell2mat({Channels.isSelected});
            obj.nCh = length(find(obj.ChSel));
            OpMode = zeros(obj.nICPInput,1);
            for i = 1 : obj.nICPInput
                OpMode(i) = Channels(i).OpMode.FPGANum;
            end
            obj.acquireFPGA.ChannelSelect = obj.ChSel;
            obj.acquireFPGA.OperationMode = OpMode;
            if isfield(obj.acquireFPGA, 'OutputEnable')
                obj.acquireFPGA.OutputEnable = false(4,1);
            end
            obj.acquireFPGA.AcquisitionMode = 1; % Continuous
            
            % Configre DMA input and output
            status = obj.acquireFPGA.DMAInput.configure(obj.DMADepth);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            status = obj.acquireFPGA.DMAInput.start();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Reserve IRQ Context
            [status, obj.Context] = obj.acquireFPGA.reserveIrq();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            status = obj.acquireFPGA.run('nowait');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Start input DMA
            status = obj.acquireFPGA.DMAInput.start();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Run FPGA code (not acquisition yet, still waiting for IRQ)
            status = obj.acquireFPGA.run('nowait');
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
        end % of function ImpactInit
        
        function status = ImpactArm(obj, nSamples, nPretrigSamples, Fs)
            % Arm impact testing (collect pretrigger samples)
            
            DataRate = CFPGA.FsToDataRate(Fs);
            
            obj.nSamples = nSamples;
            obj.nPretrigSamples = nPretrigSamples;
            
            obj.ImpactBuffer = zeros(obj.nSamples, obj.nCh);
            
            obj.acquireFPGA.DataRate = DataRate;
            obj.acquireFPGA.stop = 0;
            obj.acquireFPGA.StopSegment = 0;
            obj.acquireFPGA.AcquisitionMode = 1; % Continuous
            %             % FAKE FOR DEBUGGING
            %             obj.acquireFPGA.AcquisitionMode = 0; % FIXED
            %             obj.acquireFPGA.NumSamples = 1e5;
            
            % Start acquistion
            [status, asserted] = obj.acquireFPGA.waitOnIrq(obj.Context, 15, Inf);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            status = obj.acquireFPGA.ackIrq(asserted);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % read pretrig samples
            [status, Samples, obj.nSamplesInBuffer] =...
                obj.acquireFPGA.DMAInput.read(obj.nCh*obj.nPretrigSamples, Inf);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            
            % Copy samples into buffer and adjust sample index offset
            index = 1:obj.nPretrigSamples;
            obj.ImpactBuffer(index, :) = ...
                obj.raw2calibrated(reshape(Samples, obj.nCh, []).');
            obj.ImpactBufferIndex = obj.nPretrigSamples;
            
            status = 0;
        end % of function ImpactArm
        
        function [status, isTriggered] = ImpactWaitForTrigger(obj, trigLevel, trigCh)
            % Read a block and search for trigger event inside the block
            
            % read a block from the hardware
            TriggerBlockSize = round(obj.TriggerBlockDuration * 51200/double(obj.acquireFPGA.DataRate));
            index = obj.ImpactBufferIndex+(1:TriggerBlockSize);
            [status, block, obj.nSamplesInBuffer] =...
                obj.acquireFPGA.DMAInput.read(obj.nCh*TriggerBlockSize, Inf);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            block = obj.raw2calibrated(reshape(block, obj.nCh, []).');
            
            % determine whether trigger event has occured
            indTrig = find(abs(block(:, trigCh)) > trigLevel, 1, 'first');
            isTriggered = ~isempty(indTrig);
            if isTriggered
                % number of samples to read
                obj.nPosttrigSamples =...
                    obj.nSamples - obj.nPretrigSamples - (size(block,1) - indTrig + 1);
                if obj.nPosttrigSamples < 0
                    block = block(1 : end-abs(obj.nPosttrigSamples), :);
                    index = index(1 : end-abs(obj.nPosttrigSamples));
                    obj.nPosttrigSamples = 0;
                end
            end
            obj.ImpactBuffer(mod(index-1, obj.nSamples)+1, :) = block;
            obj.ImpactBufferIndex = obj.ImpactBufferIndex + size(block,1);
            status = 0;
        end % of function ImpactWaitForTrigger
        
        function [status, y]= ImpactAcq(obj)
            % Read post trigger samples from DMA and stop acquisition
            
            if obj.nPosttrigSamples > 0
                
                % read post trigger samples into block
                index = obj.ImpactBufferIndex + (1:obj.nPosttrigSamples);
                [status, block, obj.nSamplesInBuffer] =...
                    obj.acquireFPGA.DMAInput.read(obj.nCh*obj.nPosttrigSamples, Inf);
                if status < 0
                    error('NiHuLab:CFPGA:Hardware', status2msg(status));
                end
                
                % copy block into Buffer and adjust ImpactBufferIndex
                obj.ImpactBuffer(mod(index-1, obj.nSamples)+1,:) =...
                    obj.raw2calibrated(reshape(block, obj.nCh, []).');
                
                obj.ImpactBufferIndex = obj.ImpactBufferIndex + obj.nPosttrigSamples;
            end
            
            obj.ImpactBufferIndex = mod(obj.ImpactBufferIndex-1, obj.nSamples)+1;
            
            % rotate content of buffer
            y = circshift(obj.ImpactBuffer, -obj.ImpactBufferIndex);
            
            % Stop acquisition and flush DMA
            obj.acquireFPGA.StopSegment = 1;
            [status, ~] = CFPGA.FlushDMA(obj.acquireFPGA.DMAInput);
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
            obj.nSamplesInBuffer = 0;
        end % of function ImpactAcq
        
        function status = ImpactClose(obj)
            % Close impact testing module
            
            % Free IRQ Context if needed
            if ~isempty(obj.Context)
                status = obj.acquireFPGA.unreserveIrq(obj.Context);
                obj.Context = [];
                if status < 0
                    error('NiHuLab:CFPGA:Hardware', status2msg(status));
                end
            end
            % Close FPGA program
            status = obj.acquireFPGA.close();
            if status < 0
                error('NiHuLab:CFPGA:Hardware', status2msg(status));
            end
        end % end of function ImpactClose
    end  % of methods
    
    methods (Static = true)
        function datarate = FsToDataRate(fs)
            datarate = CNi9234Hardware.maxDataRate / fs;
        end
        
        function [status, rest] = FlushDMA(DMA)
            % Flush FPGA input DMA FIFO
            
            rest = zeros(10000,1);  % buffer for reading the rest from the DMA
            k = 0;                  % number of elements read
            while true
                [status, data, elemRemaining] = DMA.read(1, 5); % try to read 1 elem
                if status == -50400 % no element left in buffer
                    status = 0;
                    break;
                elseif status < 0   % error when reading
                    error('NiHuLab:CFPGA:Hardware', status2msg(status));
                end
                % one element read successfully, add it to the buffer
                k = k+1;
                rest(k) = data;
                % read remaining elements
                while elemRemaining
                    [status, data, elemRemaining] = DMA.read(elemRemaining, 1000);
                    if status < 0
                        error('NiHuLab:CFPGA:Hardware', status2msg(status));
                    end
                    n = length(data);
                    rest(k+(1:n)) = data;
                    k = k + length(data);
                end
            end
            rest = rest(1:k);
        end % of function FlushDMA()
        
        function Blocks = BlockSize(DMADepth, nCh, nSamples)
            maxblocksize = floor(DMADepth/2/nCh);
            nBlock = ceil(nSamples/maxblocksize);
            Blocks = floor(nSamples/nBlock) * ones(nBlock,1);
            Blocks(end) = nSamples - sum(Blocks(1:end-1));
        end % of function Blocks()
    end % of static methods
end % of Class CFPGA
