%CIMPACT Nihu.Lab module for Impact Testing
classdef CImpact < CUiTab & CChSetupSensitive
    properties
        Hardware        % handle to CHardware object
        AcqSetup        % handle to CAcqSetup object
        AcqSetupView    % handle to CAcqSetupView object
        Trigger         % handle to CTrigger object
        TriggerView     % handle to CTriggerView object
        Average         % handle to CAverage object
        AverageView     % handle to CAverageView object
        LevelMeter      % handle to CLevelMeter object
        FileSelect      % handle to File Selector object
        FileSelectView  % handle to File Select View object
        
        MeasurementResult   % measurement results are stored in this structure
        
        ExcData         % CGraphData object (excitation time history)
        ExcDataView     % handle to CGraphDataView object
        RespData        % CGraphData object (response time history)
        RespDataView    % handle to CGraphDataView object
        IRData          % CGraphData object (Impulse response)
        IRDataView      % handle to CGraphDataView object
        FRFData         % CGraphData object (FRF)
        FRFDataView     % handle to CGraphDataView object
        CohData         % CGraphData object (Coherence)
        CohDataView     % handle to CGraphDataView object
    end
    
    methods
        function obj = CImpact(ParentGroup, hardware, chsetup, acqsetup,...
                trigger, average, levmeter)
            obj = obj@CUiTab(ParentGroup, 'IMPACT_MODULE');
            obj = obj@CChSetupSensitive(chsetup);
            
            obj.Hardware = hardware;
            obj.AcqSetup = acqsetup;
            obj.Trigger = trigger;
            obj.Average = average;
            obj.LevelMeter = levmeter;
            
            obj.FileSelect = CFileSelect({'mat'});
            
            obj.IRData = CGraphData();
            obj.FRFData = CGraphData();
            obj.ExcData = CGraphData();
            obj.RespData = CGraphData();
            obj.CohData = CGraphData();
            
            obj.InitFigure();
        end
        
        function SaveCallback(obj, ~)
            Result = obj.MeasurementResult; %#ok<NASGU>
            fname = fullfile(obj.FileSelect.getFullFileName());
            save(fname, 'Result');
        end
        
        function ImpactCallback(obj, hButton)
            
            % Check whether pretrig time is smaller than acquisition time
            if (obj.AcqSetup.T < obj.Trigger.preTrigTime)
                errordlg(CBabel.getInstance().getText('IMPACT_LONG_PRETRIGGER_ERROR'), ...
                    CBabel.getInstance().getText('IMPACT_ERROR'),...
                    'modal');
                return;
            end
            
            Fs = obj.AcqSetup.Fs;
            nSamples = obj.AcqSetup.nSamples;
            nAverage = obj.Average.nAverage;
            
            tScale = (0 : nSamples-1)' / Fs;
            fScale = (0:nSamples-1)'/nSamples*Fs; % frequency scale
            index = fScale <= Fs/2; % limit fscale to fs/2
            fScale = fScale(index);
            
            % Block user interfaces
            set(gcbf, 'UserData', get(gcbf, 'CloseRequestFcn'));
            set(gcbf, 'CloseRequestFcn', '');
            obj.hParentGroup.block();
            obj.AcqSetupView.block();
            obj.AverageView.block();

            set(hButton, 'Enable', 'off');
            
            obj.Average.reset();
            
            refind = obj.ChannelSetup.getActiveReferenceIndex();
            nCh = obj.ChannelSetup.getActiveChNum();
            
            nPretrigSamples = round(Fs * obj.Trigger.preTrigTime);
            trigLevel = obj.Trigger.trigLevel;
            
            try
                y = zeros(nSamples, nAverage, nCh);
                iAver = 0;
                while iAver < nAverage
                    obj.Trigger.initialize();
                    isTriggered = false;
                    trigCh = refind;
                    status = obj.Hardware.ImpactArm(nSamples, nPretrigSamples, obj.AcqSetup.Fs);
                    obj.Trigger.arm();
                    drawnow();
                    while ~isTriggered
                        [~, isTriggered] = obj.Hardware.ImpactWaitForTrigger(trigLevel, trigCh);
                    end
                    obj.Trigger.measure();
                    drawnow();
                    [~, yy]= obj.Hardware.ImpactAcq();
                    obj.Trigger.reset();
                    
                    obj.LevelMeter.UpdateLevels(yy);
                    
                    maxy = max(abs(yy), [], 1);
                    ovld_idx = maxy > obj.ChannelSetup.getUsedChannels().getRange();
                    if any(ovld_idx)
                        hDlg = errordlg('Overload', 'Impact overload', 'modal');
                        uiwait(hDlg);
                        continue;
                    end
                    
                    iAver = iAver + 1;
                    
                    % take channel sensitiviy into account
                    y(:,iAver,:) = obj.ChannelSetup.voltage2eu(yy);

                    obj.ExcData.UpdateData(tScale, squeeze(y(:,iAver, refind)));
                    obj.RespData.UpdateData(tScale, squeeze(y(:,iAver,:)));
                    
                    drawnow();
                end % loop over averages
                
                y = obj.ChannelSetup.decompensate9234(y, obj.AcqSetup.Fs);
                
                x = y(:,:,refind);
                [H, C, h] = transferest(x, y, 'noise');
                
                H = H(index,:);
                C = C(index,:);
                
                obj.IRData.UpdateData(tScale, h);
                obj.FRFData.UpdateData(fScale, abs(H));
                obj.ExcData.UpdateData(tScale, squeeze(x(:,end)));
                obj.RespData.UpdateData(tScale, squeeze(y(:,end,:)));
                obj.CohData.UpdateData(fScale, abs(C));
                
                obj.MeasurementResult.AcqSetup = obj.AcqSetup.dcopy();
                obj.MeasurementResult.Average = obj.Average.dcopy();
                obj.MeasurementResult.Channels = obj.ChannelSetup.getUsedChannels();
                obj.MeasurementResult.ExcitationSignal = x;
                obj.MeasurementResult.ResponseSignal = y;
                obj.MeasurementResult.FrequencyResponseFunction = H;
                obj.MeasurementResult.Coherence = C;
                obj.MeasurementResult.ImpulseResponse = h;
            catch ME
                errordlg(sprintf('%s\n%s\n%s:%d', ME.identifier, ME.message, ME.stack(1).name, ME.stack(1).line), '!!! Error !!!', 'modal');
            end
            
            % Unblock User Interfaces
            set(gcbf, 'CloseRequestFcn', get(gcbf, 'UserData'));
            obj.hParentGroup.unblock();
            obj.AcqSetupView.unblock();
            obj.AverageView.unblock();
            
            set(hButton, 'Enable', 'on');
        end
    end
    
    methods (Access = private)
        function InitFigure(obj)
            obj.AcqSetupView = CAcqSetupView(obj.hContainer, obj.AcqSetup);
            obj.AverageView = CAverageView(obj.hContainer, obj.Average);
            obj.TriggerView = CTriggerView(obj.hContainer, obj.Trigger);
            h2 = CLevelMeterView(obj.hContainer, obj.LevelMeter, 100);
            obj.FileSelectView = CFileSelectView(obj.hContainer, obj.FileSelect);
            butAcq = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton',...
                'Tag', 'START_TRANSFER', ...
                'String', CBabel.getInstance().getText('START_IMPACT'), ...
                'Callback', @(src, event)ImpactCallback(obj, src));
            butSav = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton',...
                'Tag', 'SAVE_TRANSFER',...
                'String', CBabel.getInstance().getText('SAVE_TRANSFER'), ...
                'Callback', @(src, event)SaveCallback(obj, src));
            h3 = CHardwareView(obj.hContainer, obj.Hardware);
            obj.IRDataView = CGraphDataView(obj.hContainer, obj.IRData, 'IMPULSE_RESPONSE');
            obj.FRFDataView = CGraphDataView(obj.hContainer, obj.FRFData, 'FREQUENCY_RESPONSE_FUNCTION');
            obj.ExcDataView = CGraphDataView(obj.hContainer, obj.ExcData, 'EXCITATION_TIME_HISTORY');
            obj.RespDataView = CGraphDataView(obj.hContainer, obj.RespData, 'RESPONSE_TIME_HISTORY');
            obj.CohDataView = CGraphDataView(obj.hContainer, obj.CohData, 'COHERENCE_FUNCTION');
            w = get(obj.AcqSetupView.hPanel, 'position');
            w = w(3);
            lMenu = {{obj.AcqSetupView.hPanel, [-1 0]};
                {obj.TriggerView.hPanel, [-1 0]};
                {obj.AverageView.hPanel, [-1 0]};
                {obj.FileSelectView.hPanel, [-1 0]};
                {butSav, [-1 CSetupClass.itemHeight]};
                {butAcq, [-1 CSetupClass.itemHeight]};
                {h2.hPanel, [-1 -1]};
                {h3.hPanel, [-1 0]}};
            content = {{{{obj.ExcDataView.hPanel, [-1 -1]}; {obj.RespDataView.hPanel, [-1 -1]}; {obj.IRDataView.hPanel, [-1 -1]}}, [-1 -1]} {{{obj.FRFDataView.hPanel, [-1 -1]}; {obj.CohDataView.hPanel, [-1 -1]}}, [-1 -1]}};
            Layout = {{lMenu, [w, -1]} {content, [-1 -1]}};
            GuiLayout.ApplyLayout(obj.hContainer, Layout);
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
        end
    end
    
    methods
        function show(obj)
            %SHOW Called when the user opens this GUI module

            % warn if no reference channel is selected
            if isempty(obj.ChannelSetup.getActiveReferenceIndex)
                warndlg('Impact Testing without reference channel?', 'Impact testing warning', 'modal');
                return;
            end
            
            obj.LevelMeter.Reset();
            
            if obj.chSetupChanged
                Chan = obj.ChannelSetup.getUsedChannels();
                refind = obj.ChannelSetup.getActiveReferenceIndex();
                Col = {Chan.Color};
                Col = cell2mat(Col(:));

                obj.IRData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                obj.IRDataView.UpdateChannels();
                
                obj.FRFData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Frequency.textID(), CQuantity.Frequency.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                obj.FRFDataView.UpdateChannels();
                
                obj.ExcData.UpdateChannels(0,...
                    {CBabel.getInstance().getText('EXCITATION')},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    Chan(refind).getQuantText, Chan(refind).getEUText, Col(refind,:));
                obj.ExcDataView.UpdateChannels();
                
                obj.RespData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                obj.RespDataView.UpdateChannels();
                
                obj.CohData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Frequency.textID(), CQuantity.Frequency.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                
                obj.chSetupChanged = false;
            end
            
            obj.CohDataView.UpdateChannels();
            
            obj.IRDataView.unblock();
            obj.FRFDataView.unblock();
            obj.CohDataView.unblock();
            obj.ExcDataView.unblock();
            obj.RespDataView.unblock();
            
            status = obj.Hardware.ImpactInit(obj.ChannelSetup.InputChannels);
        end
        
        function hide(obj)
            %HIDE Called when the user switches to an other GUI module
            obj.IRDataView.block();
            obj.FRFDataView.block();
            obj.CohDataView.block();
            obj.ExcDataView.block();
            obj.RespDataView.block();
            
            status = obj.Hardware.ImpactClose();
        end
    end
end
