classdef CMonitor < CUiTab & CChSetupSensitive
    %CMONITOR Nihu.Lab module for signal monitoring
    properties
        Hardware        % handle to CHardware object
        AcqSetup        % handle to CAcqSetup object
        AcqSetupView    % handle to CAcqSetupView object
        LevelMeter      % handle to CLevelMeter object
        TimeData        % Time history object
        TimeDataView    % handle to CGraphDataView object
        FreqData        % Spectrum data object
        FreqDataView    % handle to CGraphDataView object
        
        isRunning       % boolean indicating whether Monitoring is on
    end
    
    methods
        function obj = CMonitor(ParentGroup, hardware, chsetup, acqsetup, levmeter)
            obj = obj@CUiTab(ParentGroup, 'MONITOR_MODULE');
            obj = obj@CChSetupSensitive(chsetup);
            
            obj.Hardware = hardware;
            obj.AcqSetup = acqsetup;
            obj.LevelMeter = levmeter;
            
            obj.TimeData = CGraphData;
            obj.FreqData = CGraphData;
            
            obj.InitFigure();
        end
        
        function MonitorCallback(obj, hButton)
            obj.isRunning = ~obj.isRunning;
            
            if ~obj.isRunning
                return;
            end
            
            set(hButton, 'String', CBabel.getInstance().getText('STOPMONITOR_BUTTON'));
            
            % block interface
            obj.hParentGroup.block();
            set(gcbf, 'UserData', get(gcbf, 'CloseRequestFcn'));
            set(gcbf, 'CloseRequestFcn', '');
            obj.AcqSetupView.block();
            
            nSamp = obj.AcqSetup.nSamples;
            tScale = (0 : nSamp-1) / obj.AcqSetup.Fs;
            fScale = (0 : nSamp-1) / nSamp * obj.AcqSetup.Fs;
            fIndex = fScale < obj.AcqSetup.Fs / 2;
            fScale = fScale(fIndex);
            
            try
                status = obj.Hardware.MonitorStart(obj.AcqSetup.Fs, nSamp);
                if status < 0
                    error('NiHuLab:hardware', 'Hardware start error');
                end
                
                while true
                    if ~obj.isRunning
                        break;
                    end
                    
                    [status, voltage] = obj.Hardware.MonitorAcq(nSamp);
                    if status < 0
                        error('NiHuLab:hardware', 'Hardware acquisition error');
                    end
                    obj.LevelMeter.UpdateLevels(voltage);
                    eu = obj.ChannelSetup.voltage2eu(voltage);
                    obj.TimeData.UpdateData(tScale, eu);
                    Eu = abs(fft(eu, [], 1));
                    obj.FreqData.UpdateData(fScale, Eu(fIndex,:));
                    
                    disp(obj.AcqSetup.Fs);
                    
                    drawnow;
                end
                
                status = obj.Hardware.MonitorStop;
                if status < 0
                    error('NiHuLab:hardware', 'Hardware closing error');
                end
            catch ME
                errordlg(sprintf('id: %s\nmsg: %s\nfile: %s\nname: %s\nline: %d', ...
                    ME.identifier, ...
                    ME.message, ...
                    ME.stack.file, ...
                    ME.stack.name, ...
                    ME.stack.line), '!!! Error !!!', 'modal');
            end
            
            set(hButton, 'String', CBabel.getInstance().getText('STARTMONITOR_BUTTON'));
            
            % unblock interface
            set(gcbf, 'CloseRequestFcn', get(gcbf, 'UserData'));
            obj.hParentGroup.unblock();
            obj.AcqSetupView.unblock();
            
            obj.isRunning = false;
        end
    end
    
    methods (Access = private)
        function InitFigure(obj)
            obj.AcqSetupView = CAcqSetupView(obj.hContainer, obj.AcqSetup);
            h2 = CLevelMeterView(obj.hContainer, obj.LevelMeter, 100);
            butAcq = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton',...
                'Tag', 'STARTMONITOR_BUTTON',...
                'String', CBabel.getInstance().getText('STARTMONITOR_BUTTON'), ...
                'Callback', @(src, event)MonitorCallback(obj, src));
            h3 = CHardwareView(obj.hContainer, obj.Hardware);
            obj.TimeDataView = CGraphDataView(obj.hContainer, obj.TimeData, 'MONITOR_TIME_HISTORIES');
            obj.FreqDataView = CGraphDataView(obj.hContainer, obj.FreqData, 'MONITOR_FREQUENCY_CONTENTS');
            w = get(obj.AcqSetupView.hPanel, 'position');
            w = w(3);
            lMenu = {
                {obj.AcqSetupView.hPanel, [-1 0]}
                {butAcq, [-1 CSetupClass.itemHeight]}
                {h2.hPanel, [-1 -1]}
                {h3.hPanel, [-1 0]}
                };
            rMenu = {
                {obj.TimeDataView.hPanel, [-1 -1]}
                {obj.FreqDataView.hPanel, [-1 -1]}
                };
            Layout = {{lMenu, [w, -1]} {rMenu, [-1 -1]}};
            GuiLayout.ApplyLayout(obj.hContainer, Layout);
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
        end
    end
    
    methods
        function show(obj)
            %SHOW function called each time before the module is opened

            obj.isRunning = false;
            
            obj.LevelMeter.Reset();
            
            if obj.chSetupChanged
                Chan = obj.ChannelSetup.getUsedChannels();
                Col = {Chan.Color};
                Col = cell2mat(Col(:));
                
                obj.TimeData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Time.textID(), CQuantity.Time.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                obj.TimeDataView.UpdateChannels();
                
                obj.FreqData.UpdateChannels(cell2mat({Chan.PhysChNr}),...
                    {Chan.Name},...
                    CQuantity.Frequency.textID(), CQuantity.Frequency.EU,...
                    Chan.getQuantText(), Chan.getEUText(), Col);
                obj.FreqDataView.UpdateChannels();
                
                obj.chSetupChanged = false;
            end
            
            obj.TimeDataView.unblock();
            obj.FreqDataView.unblock();
            
            try
                status = obj.Hardware.MonitorInit(obj.ChannelSetup.InputChannels);
                if status < 0
                    error('NiHuLab:hardware', 'Hardware init error');
                end
            catch ME
                errordlg(sprintf('%s\n%s', ME.identifier, ME.message), '!!! Error !!!', 'modal');
            end
        end
        
        function hide(obj)
            %SHOW function called each time before the module is closed
            
            obj.TimeDataView.block();
            obj.FreqDataView.block();
            status = obj.Hardware.MonitorClose();
        end
    end
end
